#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static int byteCounter = 0;

#define BUFFERSIZE 1024*10

typedef struct {
	char *hdr;
	char *body;
	char *index;
} record_t;

record_t **newRecord(record_t **records, record_t **newRecord, char *hdr) {
	int count = 0;
	record_t **retval;

	if (records)
		for (count = 0; records[count]; count++)
			;

	if (count) {
		retval = (record_t **) realloc(records, sizeof(record_t *) * (count + 2));
	} else {
		retval = (record_t **) malloc(sizeof(record_t *) * (count + 2));
	}
	count += 2;

	retval[count-1] = 0;
	*newRecord = retval[count - 2] = (record_t *) malloc(sizeof(record_t));
	(*newRecord)->hdr = NULL;
	(*newRecord)->body = NULL;
	(*newRecord)->index = NULL;
	return retval;
}

void ignoreMultipleEOL() {
	char c;

	while ((c = getchar()) != EOF && (c == '\n' || c == '\015'))
		byteCounter++;
	if (c != EOF) {
		ungetc(c, stdin);
	}
}	

int compareRecords(record_t **a, record_t **b) {
	return(strcmp((*a)->index, (*b)->index));
}

int main(int argc, char *argv[]) {
	char buffer[BUFFERSIZE], *b;
	char c;
	record_t **records = NULL, *thisRecord = NULL;
	enum state_e { in_hdr, in_body, in_list, in_index };
	enum state_e state = in_hdr;
	int count;
	
	buffer[0] = 0;
	b = buffer;

	while ((c = getchar()) != EOF) { /* I don't care about EOF character so no feof() */
		byteCounter++;
		switch (c) {
		case '\225':
			if (state != in_list)
				b += strlen(strcat(b, "<UL>\n"));
			b += strlen(strcat(b, "<LI>"));
			while ((c = getchar()) != '\015' && c != '\n' && c != '|' && c != EOF) {
				byteCounter++;
				*b++ = c;
			}
			*b++ = '\n';
			*b = 0;
			state = in_list;
			if (c == '|') {
				ungetc(c, stdin);
				byteCounter--;
			} else
				ignoreMultipleEOL();
			break;

		case '\015':
			if (state == in_list) {
				b += strlen(strcat(b, "</UL>\n"));
				state = in_body;
			}
			b += strlen(strcat(b, "<P>\n"));
			ignoreMultipleEOL();
			break;

		case '|':
			if (state == in_hdr) {
				records = newRecord(records, &thisRecord, buffer);
				thisRecord->hdr = strdup(buffer);
				b = buffer;
				*b = 0;
				state = in_body;
				continue;
			}
			if (state == in_list) {
				b += strlen(strcat(b, "</UL>\n"));
				state = in_body;
			}
			if (state == in_body) {
				thisRecord->body = strdup(buffer);
				b = buffer;
				*b = 0;
				state = in_index;
				break;
			}
			fprintf(stderr, "Got a | in index!: at byte %d\n", byteCounter);
			return 1;

		case '#':
			if (state != in_index) {
				fprintf(stderr, "Got a # outside an index!: at byte %d\n", byteCounter);
				return 1;
			}
			thisRecord->index = strdup(buffer);
			b = buffer;
			*b = 0;
			state = in_hdr;
			break;

		default:
			if (state == in_list) {
				b += strlen(strcat(b, "</UL>\n"));
				state = in_body;
			}	
			*b++ = c;
			*b = 0;
			break;
		}
	}

	for (count = 0; (thisRecord = records[count]); count++)
		;
	qsort(records, count, sizeof(record_t *), compareRecords);

	for (count = 0; (thisRecord = records[count]); count++) {
		printf("_H1(%s)\n", thisRecord->hdr);
		printf("%s\n", thisRecord->body);
	}
	return 0;
}


/* For emacs: */
/* Local Variables: */
/* fill-column: 70 */
/* eval:(setq filename (substring buffer-file-name (string-match "[a-zA-Z0-9_.]+$" buffer-file-name))) */
/* eval:(setq basename (substring filename 0 (string-match "\\." filename))) */
/* eval:(setq compile-command (concat "cc -Wall -g -o " basename " -DDEBUG=1 -UNDEBUG " basename ".c")) */
/* End: */
